#include "util.h"
#include "flash-rom.h"

extern unsigned int guiMode16MB;
extern flash_dev *gpFlash;

#define REG_AHBC_BASE       0x90100000
#define REG_SMC_BASE        0x90200000
#define REG_PCU_BASE        0x98100000
#define REG_AHBC_BASE_16    0x00E00000
#define REG_SMC_BASE_16     0x00E01000
#define REG_PCU_BASE_16     0x00F01000
//#define REG_DMAC_BASE     0x90400000
//#define REG_DMAC_BASE_16  0x00E03000

unsigned int platform_init_4gb(void);
unsigned int platform_init_16mb(void);
unsigned int platform_get_version_id(void);

unsigned int platform_init(void)
{
  unsigned int Ret;

  if(guiMode16MB == 0)
    Ret = platform_init_4gb();
  else
    Ret = platform_init_16mb();

  return Ret;
}

/* XC5 initial function */
unsigned int platform_init_4gb(void)
{
  const unsigned int AHBC_BASE = REG_AHBC_BASE;
  const unsigned int SMC_BASE  = REG_SMC_BASE;
  unsigned int ahbc_control_reg;
  unsigned int base_address_high; // [31:20]
  unsigned int base_address_low;  // [27:15]
  unsigned int smc_bank0_reg;

  platform_get_version_id();
  /* read AHBC control register. If bit0 is 0, un-remap. If bit0 is 1, remap. */
  ahbc_control_reg = inw(AHBC_BASE + 0x88);
  if ((ahbc_control_reg & 0x1) == 0)
  {
    /* Write memory bank 0 configuration register */
    /* Disable write-protect, set BNK_SIZE to 64MB, set BNK_MBW to 32 */
		smc_bank0_reg = inw(SMC_BASE);
		/* limit BNK_SIZE = 32MB */
		smc_bank0_reg = (smc_bank0_reg & ~0xf0) | 0x50;
		/* disable write protection */
		smc_bank0_reg = (smc_bank0_reg & ~0x800);
		outw(SMC_BASE, smc_bank0_reg);
		
		/*outw(SMC_BASE+4, 0x00153153);*/
    return 0; // un-remap, base is 0x0
  }

  base_address_high = inw(AHBC_BASE + 0x10);
  base_address_low = inw(SMC_BASE);
  return (base_address_high & 0xFFF00000) | (base_address_low & 0x0FFF8000);
}

/* for 24 bit platform */
unsigned int platform_init_16mb(void)
{
  const unsigned int AHBC_BASE = REG_AHBC_BASE_16;
  const unsigned int SMC_BASE  = REG_SMC_BASE_16;
  unsigned int ahbc_control_reg;
  unsigned int base_address_high; // [31:20]
  unsigned int base_address_low;  // [27:15]
  unsigned int smc_bank0_reg;

  /* read AHBC control register. If bit0 is 0, un-remap. If bit0 is 1, remap. */
  ahbc_control_reg = inw(AHBC_BASE + 0x88);
  //fprintf (log, "ahbc control: 0x%x\n", ahbc_control_reg);
  if ((ahbc_control_reg & 0x1) == 0)
  {
    /* Write memory bank 0 configuration register */
    /* Disable write-protect, set BNK_SIZE to 32MB, set BNK_MBW to 32 */
		smc_bank0_reg = inw(SMC_BASE);
		/* limit BNK_SIZE = 1MB */
		smc_bank0_reg = (smc_bank0_reg & ~0xf0);
		/* disable write protection */
		smc_bank0_reg = (smc_bank0_reg & ~0x800);
		outw(SMC_BASE, smc_bank0_reg);

    return 0; // un-remap, base is 0x0
  }

  base_address_high = inw(AHBC_BASE + 0x10);
  base_address_low = inw(SMC_BASE);

  platform_get_version_id();
  gpFlash->flash_chipsize = 0x100000;
  //fprintf (log, "base: 0x%x\n", ((base_address_high & 0xFFF00000) | (base_address_low & 0x0FFF8000)) >> 8);
  return ((base_address_high & 0xFFF00000) | (base_address_low & 0x0FFF8000)) & 0x00FFFFFF;
}

unsigned int platform_get_version_id(void)
{
  unsigned int id1;

  if(guiMode16MB == 0)
    id1 = inw(REG_PCU_BASE + 0x00);
  else
    id1 = inw(REG_PCU_BASE_16 + 0x00);
  printf("PMU_VER_ID = %08x \n", id1);
  return 0;
}
